/* ASN.1 BER/DER/CER parsing state machine definitions
 *
 * Copyright (C) 2012 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version
 * 2 of the Licence, or (at your option) any later version.
 */

#ifndef __ASN1_MACHINE_H
#define __ASN1_MACHINE_H

#include <stdlib.h>

/* Class */
enum asn1_class {
	ASN1_UNIV	= 0,	/* Universal */
	ASN1_APPL	= 1,	/* Application */
	ASN1_CONT	= 2,	/* Context */
	ASN1_PRIV	= 3	/* Private */
};
#define ASN1_CLASS_BITS	0xc0


enum asn1_method {
	ASN1_PRIM	= 0,	/* Primitive */
	ASN1_CONS	= 1	/* Constructed */
};
#define ASN1_CONS_BIT	0x20

/* Tag */
enum asn1_tag {
	ASN1_EOC	= 0,	/* End Of Contents or N/A */
	ASN1_BOOL	= 1,	/* Boolean */
	ASN1_INT	= 2,	/* Integer */
	ASN1_BTS	= 3,	/* Bit String */
	ASN1_OTS	= 4,	/* Octet String */
	ASN1_NULL	= 5,	/* Null */
	ASN1_OID	= 6,	/* Object Identifier  */
	ASN1_ODE	= 7,	/* Object Description */
	ASN1_EXT	= 8,	/* External */
	ASN1_REAL	= 9,	/* Real float */
	ASN1_ENUM	= 10,	/* Enumerated */
	ASN1_EPDV	= 11,	/* Embedded PDV */
	ASN1_UTF8STR	= 12,	/* UTF8 String */
	ASN1_RELOID	= 13,	/* Relative OID */
	/* 14 - Reserved */
	/* 15 - Reserved */
	ASN1_SEQ	= 16,	/* Sequence and Sequence of */
	ASN1_SET	= 17,	/* Set and Set of */
	ASN1_NUMSTR	= 18,	/* Numerical String */
	ASN1_PRNSTR	= 19,	/* Printable String */
	ASN1_TEXSTR	= 20,	/* T61 String / Teletext String */
	ASN1_VIDSTR	= 21,	/* Videotex String */
	ASN1_IA5STR	= 22,	/* IA5 String */
	ASN1_UNITIM	= 23,	/* Universal Time */
	ASN1_GENTIM	= 24,	/* General Time */
	ASN1_GRASTR	= 25,	/* Graphic String */
	ASN1_VISSTR	= 26,	/* Visible String */
	ASN1_GENSTR	= 27,	/* General String */
	ASN1_UNISTR	= 28,	/* Universal String */
	ASN1_CHRSTR	= 29,	/* Character String */
	ASN1_BMPSTR	= 30,	/* BMP String */
	ASN1_LONG_TAG	= 31	/* Long form tag */
};


typedef int (*asn1_action_t)(void *context, unsigned state_index,
			     unsigned char tag, /* In case of ANY type */
			     const void *value, size_t vlen);

extern asn1_action_t asn1_action_table[];

enum asn1_opcode {
	/* The tag-matching ops come first and the odd-numbered slots
	 * are for OR_SKIP ops.
	 */
	ASN1_OP_MATCH			= 0x00,
	ASN1_OP_MATCH_OR_SKIP		= 0x01,
	ASN1_OP_MATCH_ACT		= 0x02,
	ASN1_OP_MATCH_ACT_OR_SKIP	= 0x03,
	ASN1_OP_MATCH_JUMP		= 0x04,
	ASN1_OP_MATCH_JUMP_OR_SKIP	= 0x05,
	ASN1_OP_MATCH_ANY		= 0x06,
	__RESERVED__ANY_SKIP_1		= 0x07,
	ASN1_OP_MATCH_ANY_ACT		= 0x08,
	/* Everything before here matches unconditionally */
#define ASN1_OP__MATCHES_UNCONDITIONALLY ASN1_OP_MATCH_ANY_ACT
#define ASN1_OP_MATCH__SKIP		  0x01

	ASN1_OP_COND_MATCH_OR_SKIP	= 0x09,
	ASN1_OP_COND_MATCH_ANY		= 0x0a,
	ASN1_OP_COND_MATCH_JUMP_OR_SKIP	= 0x0b,
	ASN1_OP_COND_MATCH_ANY_ACT	= 0x0c,
	ASN1_OP_COND_MATCH_ACT_OR_SKIP	= 0x0d,
	/* Everything before here will want a tag */
#define ASN1_OP__MATCHES_TAG ASN1_OP_COND_MATCH_ANY_ACT

	ASN1_OP_COND_FAIL		= 0x0e,
	ASN1_OP_COMPLETE		= 0x0f,

	/* The following eight have bit 0 -> SET, 1 -> OF, 2 -> ACT */
	ASN1_OP_END_SEQ			= 0x10,
	ASN1_OP_END_SET			= 0x11,
	ASN1_OP_END_SEQ_OF		= 0x12,
	ASN1_OP_END_SET_OF		= 0x13,
	ASN1_OP_END_SEQ_ACT		= 0x14,
	ASN1_OP_END_SET_ACT		= 0x15,
	ASN1_OP_END_SEQ_OF_ACT		= 0x16,
	ASN1_OP_END_SET_OF_ACT		= 0x17,
#define ASN1_OP_END__SET		  0x01
#define ASN1_OP_END__OF			  0x02
#define ASN1_OP_END__ACT		  0x04

	ASN1_OP_ACT			= 0x18,
	ASN1_OP_RETURN			= 0x19,

	ASN1_OP__NR
};

#define _tag(CLASS, CP, TAG) ((ASN1_##CLASS << 6) | (ASN1_##CP << 5) | ASN1_##TAG)
#define _tagn(CLASS, CP, TAG) ((ASN1_##CLASS << 6) | (ASN1_##CP << 5) | TAG)
#define _jump_target(N) (N)
#define _action(N) (N)

struct asn1_decoder {
	const unsigned char *machine;
	size_t machlen;
	const asn1_action_t *actions;
};

extern int asn1_decoder(const struct asn1_decoder *decoder,
			void *context,
			const unsigned char *data,
			size_t datalen);

#endif /* __ASN1_MACHINE_H */
